# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .dashboard_public import DashboardPublic


class DashboardPagePublic(UniversalBaseModel):
    content: typing.Optional[typing.List[DashboardPublic]] = None
    page: typing.Optional[int] = None
    size: typing.Optional[int] = None
    total: typing.Optional[int] = None
    sortable_by: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
