# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .feedback_score_batch_item_thread_source import FeedbackScoreBatchItemThreadSource


class FeedbackScoreBatchItemThread(UniversalBaseModel):
    project_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    If null, the default project is used
    """

    project_id: typing.Optional[str] = None
    name: str
    category_name: typing.Optional[str] = None
    value: float
    reason: typing.Optional[str] = None
    source: FeedbackScoreBatchItemThreadSource
    author: typing.Optional[str] = None
    thread_id: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
